% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Dr. Jos Ernesto Rayas-Snchez                                 April 10, 2019
% Research Group on Computer-Aided Engineering of Circuits and Systems (CAECAS)
% Department of Electronics, Systems and Informatics                      ITESO
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%  Ideal Two-Section Impedance Transformer Simulated in Matlab (coarse model)
%
% This function calculates the magnitude and phase of S11 of a two-section 
% impedance transformer with ideal transmission lines. A reference 
% impedance of Zo ohms is used.
%
% Usage: [psi,R] = ZtranTL_2sec_c_Matlab(P)
%        P(1) = Zo; Reference impedance (ohms).
%        P(2) = ZL; Load impedance (ohms).
%        P(3) = Z1; TL1 characteristic impedance (ohms), connected to source.
%        P(4) = Z2; TL2 characteristic impedance (ohms), connected to load.
%        P(5) = epsr; Effective dielectric constant for both TLs.
%        P(6) = f0; Reference frequency for wavelength on TLs (GHz).
%        P(7) = L1; Electrical length of TL1 (degrees), connected to source.
%        P(8) = L2; Electrical length of TL2 (degrees), connected to load.
%        P(9) = IF; Initial frequency (GHz).
%        P(10) = FF; Final frequency (GHz).
%        P(11) = FP; Number of frequency samples.
%        psi: column vector containing the simulated frequency values (GHz).                    
%        R: matrix containing the circuit response values (S-parameters).

function [psi,R] = ZtranTL_2sec_c_Matlab(P)

% Reading Parameters
Zo = P(1);     % Reference impedance (ohms).
ZL = P(2);     % Load impedance (ohms).
Z1 = P(3);     % TL1 characteristic impedance (ohms), connected to source.
Z2 = P(4);     % TL2 characteristic impedance (ohms), connected to load.
epsr = P(5);   % Effective dielectric constant of both TLs.
f0 = P(6);     % Reference frequency for wavelength on TLs (GHz).
L1 = P(7);     % Electrical length of TL1 (degrees), connected to source.
L2 = P(8);     % Electrical length of TL2 (degrees), connected to load.
IF = P(9);     % Initial frequency (GHz).
FF = P(10);    % Final frequency (GHz).
FP = P(11);    % Number of frequency samples.

% Physical Constants
eps0 = 8.854e-12;      % Permittivity of free-space (F/m).
miu0 = 4*pi*1e-7;      % Permeability of free-space (H/m).
c = 1/sqrt(eps0*miu0); % Speed of light in free-space (m/s).

% Preliminary Calculations
vp = c/sqrt(epsr);     % Phase velocity on both TLs (m/s). 
lambda0 = vp/(f0*1e9); % Wavelength at reference frequency on both TLs (m).
beta0 = 2*pi/lambda0;  % Propagation constant at reference frequency (1/m).
Ls = [L1 L2];     % Electrical lengths (degrees).
Ls = Ls*(pi/180); % Converting electrical length from degrees to radians.
Ls = Ls/beta0;    % Physical length of the TLs (m).
L1m = Ls(1);      % Physical length of first TL (m), connected to the source.
L2m = Ls(2);      % Physical length of second TL (m), connected to the load.

% Calculating Responses
j = sqrt(-1);
f = linspace(IF,FF,FP); % Row vector of frequencies (GHz).
lambda = vp./(f*1e9);   % Wavelength at each frequency (m).
beta = 2*pi./lambda;    % Propagation constant at each frequency (1/m).
betaL1 = beta*L1m;      % TL1 electrical length at each frequency (rad).
betaL2 = beta*L2m;      % TL2 electrical length at each frequency (rad).
tanbl1 = tan(betaL1);
tanbl2 = tan(betaL2);
Zin1 = Z2*(ZL + j*Z2*tanbl1)./(Z2 + j*ZL*tanbl1);
Zin = Z1*(Zin1 + j*Z1*tanbl2)./(Z1 + j*Zin1.*tanbl2);
S11cx = (Zin-Zo)./(Zin+Zo);
mS11 = abs(S11cx);
pS11 = (180/pi)*angle(S11cx);

psi = f'; 
R(:,1) = mS11';
R(:,2) = pS11';
