% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Dr. Jos Ernesto Rayas-Snchez                                 April 10, 2019
% Research Group on Computer-Aided Engineering of Circuits and Systems (CAECAS)
% Department of Electronics, Systems and Informatics                      ITESO
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Non-Ideal Two-Section Impedance Transformer Simulated in Matlab (fine model)
%
% This function calculates the magnitude and phase of S11 of a two-section 
% impedance transformer with capacitively loaded ideal transmission lines.
% A reference impedance of 50 Ohms is used. 
%
% Usage: [psi,R] = ZtranTL_2sec_f_Matlab(P,JFN)
%        P(1) = Zo; Reference impedance (ohms).
%        P(2) = ZL; Load impedance (ohms).
%        P(3) = Z1; TL1 characteristic impedance (ohms), connected to source.
%        P(4) = Z2; TL2 characteristic impedance (ohms), connected to load.
%        P(5) = epsr; Effective dielectric constant for both TLs.
%        P(6) = f0; Reference frequency for wavelength on TLs (GHz).
%        P(7) = C1; Parasitic capacitor 1 (pF), connected to the source.
%        P(8) = C2; Parasitic capacitor 2 (pF), connected to transition.
%        P(9) = C3; Parasitic capacitor 3 (pF), connected to the load.
%        P(10) = L1; Electrical length of TL1 (degrees), connected to source.
%        P(11) = L2; Electrical length of TL2 (degrees), connected to load.
%        P(12) = IF; Initial frequency (GHz).
%        P(13) = FF; Final frequency (GHz).
%        P(14) = FP; Number of frequency samples.
%        psi: column vector containing the simulated frequency values (GHz).                    
%        R: matrix containing the circuit response values (S-parameters).

function [psi,R] = ZtranTL_2sec_f_Matlab(P)

% Reading Parameters
Zo = P(1);     % Reference impedance (ohms).
ZL = P(2);     % Load impedance (ohms).
Z1 = P(3);     % TL1 characteristic impedance (ohms), connected to source.
Z2 = P(4);     % TL2 characteristic impedance (ohms), connected to load.
epsr = P(5);   % Effective dielectric constant of both TLs.
f0 = P(6);     % Reference frequency for wavelength on TLs (GHz).
C1 = P(7);     % Parasitic capacitor 1 (pF).
C2 = P(8);     % Parasitic capacitor 2 (pF).
C3 = P(9);     % Parasitic capacitor 3 (pF).
L1 = P(10);    % Electrical length of TL1 (degrees), connected to source.
L2 = P(11);    % Electrical length of TL2 (degrees), connected to load.
IF = P(12);    % Initial frequency (GHz).
FF = P(13);    % Final frequency (GHz).
FP = P(14);    % Number of frequency samples.

% Physical Constants
eps0 = 8.854e-12;      % Permittivity of free-space (F/m).
miu0 = 4*pi*1e-7;      % Permeability of free-space (H/m).
c = 1/sqrt(eps0*miu0); % Speed of light in free-space (m/s).

% Preliminary Calculations
vp = c/sqrt(epsr);     % Phase velocity on both TLs (m/s). 
lambda0 = vp/(f0*1e9); % Wavelength at reference frequency on both TLs (m).
beta0 = 2*pi/lambda0;  % Propagation constant at reference frequency (1/m).
Ls = [L1 L2];     % Electrical lengths (degrees).
Ls = Ls*(pi/180); % Converting electrical length from degrees to radians.
Ls = Ls/beta0;    % Physical length of the TLs (m).
L1m = Ls(1);      % Physical length of first TL (m), connected to the source.
L2m = Ls(2);      % Physical length of second TL (m), connected to the load.

% Calculating Responses
j = sqrt(-1);
f = linspace(IF,FF,FP);  % Row vector of frequencies (GHz).
w = 2*pi*f*1e9;          % Row vector of angular frequencies (rad/s).
Zc1 = 1./(j*w*C1*1e-12); % Capacitor impedances.
Zc2 = 1./(j*w*C2*1e-12);
Zc3 = 1./(j*w*C3*1e-12);
lambda = vp./(f*1e9);    % Wavelength at each frequency (m).
beta = 2*pi./lambda;     % Propagation constant at each frequency (1/m).
betaL1 = beta*L1m;       % TL1 electrical length at each frequency (rad).
betaL2 = beta*L2m;       % TL2 electrical length at each frequency (rad).
tanbl1 = tan(betaL1);
tanbl2 = tan(betaL2);
Zp1 = (ZL*Zc3)./(ZL+Zc3);
Zin1 = Z2*(Zp1 + j*Z2*tanbl1)./(Z2 + j*Zp1.*tanbl1);
Zp2 = (Zin1.*Zc2)./(Zin1+Zc2);
Zin2 = Z1*(Zp2 + j*Z1*tanbl2)./(Z1 + j*Zp2.*tanbl2);
Zin = (Zin2.*Zc1)./(Zin2+Zc1);
S11cx = (Zin-Zo)./(Zin+Zo);
mS11 = abs(S11cx);
pS11 = (180/pi)*angle(S11cx);

psi = f'; 
R(:,1) = mS11';
R(:,2) = pS11';
